"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var WebFont = require('webfontloader');
require("rxjs/add/operator/debounceTime");
require("rxjs/add/operator/distinctUntilChanged");
var Rx_1 = require('rxjs/Rx');
var forms_1 = require('@angular/forms');
var core_1 = require('@angular/core');
var angular2_perfect_scrollbar_1 = require('angular2-perfect-scrollbar');
var font_picker_service_1 = require('./font-picker.service');
var interfaces_1 = require('./interfaces');
var DialogComponent = (function () {
    function DialogComponent(el, service) {
        this.el = el;
        this.service = service;
        this.styles = [];
        this.fontAmount = 5;
        this.loadedFonts = 0;
        this.presetFonts = [];
        this.googleFonts = [];
        this.currentFonts = [];
        this.dialogArrowSize = 10;
        this.dialogArrowOffset = 15;
        this.searchTerm = new forms_1.FormControl('');
        this.config = {
            suppressScrollX: true
        };
        this.loading = true;
        this.selectedFont = false;
        this.presetVisible = true;
    }
    DialogComponent.prototype.ngOnInit = function () {
        var _this = this;
        this.searchTerm
            .valueChanges
            .debounceTime(500)
            .distinctUntilChanged()
            .subscribe(function (text) {
            if (!text) {
                _this.presetVisible = true;
                _this.listLabel = "Popular fonts";
            }
            else {
                _this.presetVisible = false;
                _this.listLabel = "Search results";
            }
            _this.searchGoogleFonts(text);
        });
        Rx_1.Observable.fromEvent(this.el.nativeElement, 'ps-y-reach-end')
            .debounceTime(150)
            .subscribe(function () { return _this.loadMoreFonts(); });
        this.testContainer = document.createElement('span');
        this.testContainer.innerHTML = Array(100).join('wi');
        this.testContainer.style.cssText = [
            'position:absolute',
            'width:auto',
            'font-size:128px',
            'left:-99999px'
        ].join(' !important;');
        this.listenerResize = function () { return _this.onResize(); };
        this.listenerMouseDown = function (event) { return _this.onMouseDown(event); };
        this.openFontPicker();
    };
    DialogComponent.prototype.setDialog = function (instance, elementRef, font, fpPosition, fpPositionOffset, fpPositionRelativeToArrow, fpPresetLabel, fpPresetFonts, fpUploadButton, fpUploadButtonClass, fpUploadButtonText, fpStyleSelect, fpSizeSelect, fpCancelButton, fpCancelButtonClass, fpCancelButtonText, fpHeight, fpWidth) {
        var _this = this;
        this.listLabel = "Loading fonts...";
        this.directiveInstance = instance;
        this.directiveElementRef = elementRef;
        this.updateDialog(font, fpPosition, fpPositionOffset, fpPositionRelativeToArrow, fpPresetLabel, fpPresetFonts, fpUploadButton, fpUploadButtonClass, fpUploadButtonText, fpStyleSelect, fpSizeSelect, fpCancelButton, fpCancelButtonClass, fpCancelButtonText, fpHeight, fpWidth);
        this.service.getAllFonts('popularity').subscribe(function (data) {
            _this.loading = false;
            _this.googleFonts = data.items.map(function (font) { return _this.convertGoogleFont(font); });
            var searchFont = _this.findFont(_this.initialFont.family, true);
            if (searchFont) {
                _this.font.files = searchFont.files;
                _this.font.styles = searchFont.styles;
                _this.loadGoogleFonts([_this.font]);
            }
            var openSans = _this.googleFonts.find(function (font) { return font.family == "Open sans"; });
            _this.loadGoogleFonts([openSans]);
        }, function (err) { return console.log(err); });
    };
    DialogComponent.prototype.updateDialog = function (font, fpPosition, fpPositionOffset, fpPositionRelativeToArrow, fpPresetLabel, fpPresetFonts, fpUploadButton, fpUploadButtonClass, fpUploadButtonText, fpStyleSelect, fpSizeSelect, fpCancelButton, fpCancelButtonClass, fpCancelButtonText, fpHeight, fpWidth) {
        this.font = font;
        this.styles = font.styles;
        this.initialFont = new interfaces_1.Font(font);
        this.fpPosition = fpPosition;
        this.fpPositionOffset = parseInt(fpPositionOffset);
        if (!fpPositionRelativeToArrow) {
            this.dialogArrowOffset = 0;
        }
        this.fpPresetLabel = fpPresetLabel;
        this.fpPresetFonts = fpPresetFonts;
        this.fpStyleSelect = fpStyleSelect;
        this.fpSizeSelect = fpSizeSelect;
        this.fpCancelButton = fpCancelButton;
        this.fpCancelButtonText = fpCancelButtonText;
        this.fpCancelButtonClass = fpCancelButtonClass;
        this.fpUploadButton = fpUploadButton;
        this.fpUploadButtonText = fpUploadButtonText;
        this.fpUploadButtonClass = fpUploadButtonClass;
        this.fpWidth = parseInt(fpWidth);
        this.fpHeight = parseInt(fpHeight);
        this.searchTerm.reset({ disabled: (this.fpPresetFonts.length > 0) });
        this.setDisplayedFontSource();
    };
    DialogComponent.prototype.openFontPicker = function () {
        if (!this.open) {
            this.setDialogPosition();
            this.searchTerm.setValue('');
            window.addEventListener('resize', this.listenerResize);
            document.addEventListener('mousedown', this.listenerMouseDown);
            this.open = true;
        }
    };
    DialogComponent.prototype.closeFontPicker = function () {
        this.open = false;
        window.removeEventListener('resize', this.listenerResize);
        document.removeEventListener('mouseup', this.listenerMouseDown);
    };
    DialogComponent.prototype.isFontAvailable = function (font) {
        if (!this.testWidth) {
            this.testContainer.style.fontFamily = 'monospace';
            document.body.appendChild(this.testContainer);
            this.testWidth = this.testContainer.clientWidth;
            document.body.removeChild(this.testContainer);
        }
        this.testContainer.style.fontFamily = font.family + ', monospace';
        document.body.appendChild(this.testContainer);
        var width = this.testContainer.clientWidth;
        document.body.removeChild(this.testContainer);
        return width != this.testWidth;
    };
    DialogComponent.prototype.getPresetFonts = function () {
        var _this = this;
        var presetFonts = [];
        if (this.googleFonts && this.fpPresetFonts && this.fpPresetFonts.length) {
            this.fpPresetFonts.forEach(function (font) {
                var fontClass = _this.findFont(font, true);
                if (!fontClass) {
                    fontClass = new interfaces_1.Font({
                        family: font,
                        size: 14,
                        style: "regular",
                        styles: ["regular", "italic", "700", "700italic"]
                    });
                }
                presetFonts.push(fontClass);
            });
            this.presetFonts = presetFonts;
            return presetFonts;
        }
    };
    DialogComponent.prototype.setDisplayedFontSource = function () {
        if (this.fpPresetFonts && this.fpPresetFonts.length) {
            this.setCurentFonts(this.getPresetFonts());
        }
        else {
            this.setCurentFonts(this.googleFonts);
        }
    };
    DialogComponent.prototype.setCurentFonts = function (target) {
        var _this = this;
        if (target != this.currentFonts) {
            this.currentFonts = target;
            this.loadedFonts = this.fontAmount;
            var initialFonts = this.currentFonts.slice(0, this.fontAmount);
            this.loadGoogleFonts(initialFonts);
            setTimeout(function () {
                _this.scrollbar.scrollTo(0);
            }, 0);
        }
    };
    DialogComponent.prototype.findFont = function (searchVal, exactMatch) {
        if (exactMatch === void 0) { exactMatch = false; }
        return this.findFonts(searchVal, exactMatch)[0];
    };
    DialogComponent.prototype.findFonts = function (searchVal, exactMatch) {
        if (exactMatch === void 0) { exactMatch = false; }
        searchVal = searchVal.toLowerCase();
        var fullmatchFonts = [];
        var candidateFonts = [];
        this.googleFonts.forEach(function (font) {
            if (searchVal === font.family.toLowerCase()) {
                fullmatchFonts.push(font);
                return;
            }
            if (exactMatch == false && font.family.toLowerCase().indexOf(searchVal) > -1) {
                candidateFonts.push(font);
            }
        });
        var resultFonts = fullmatchFonts.concat(candidateFonts);
        return resultFonts;
    };
    DialogComponent.prototype.loadMoreFonts = function () {
        var _this = this;
        if (this.loading == false && this.loadedFonts < this.currentFonts.length) {
            var moreFonts = this.currentFonts.slice(this.loadedFonts, this.loadedFonts + this.fontAmount);
            this.loadGoogleFonts(moreFonts);
            this.loadedFonts += moreFonts.length;
            setTimeout(function () { _this.scrollbar.update(); }, 0);
        }
    };
    DialogComponent.prototype.loadGoogleFonts = function (fonts) {
        var _this = this;
        fonts.slice(0, this.fontAmount).forEach(function (font) {
            if (font && font.files && !_this.isFontAvailable(font)) {
                var style = font.styles.indexOf("regular") > -1 ? '' : ":" + font.styles.find(function (x) { return !isNaN(x); });
                try {
                    WebFont.load({
                        google: {
                            families: [font.family + ":" + style]
                        }
                    });
                }
                catch (e) {
                    console.warn("Problem with loading font:", font);
                }
            }
        });
    };
    DialogComponent.prototype.searchGoogleFonts = function (value) {
        if (!value) {
            this.onSearchReset();
            return;
        }
        value = value.toLowerCase();
        var searchResult = Array();
        if (this.googleFonts) {
            this.loadedFonts = this.fontAmount;
            searchResult = this.findFonts(value, false);
            this.setCurentFonts(searchResult);
        }
    };
    DialogComponent.prototype.convertGoogleFont = function (font) {
        var convertedFont = new interfaces_1.Font({
            family: font.family,
            styles: font.variants,
            files: font.files,
            style: null,
            size: 14
        });
        return convertedFont;
    };
    DialogComponent.prototype.onResize = function () {
        if (this.position === 'fixed') {
            this.setDialogPosition();
        }
    };
    DialogComponent.prototype.onMouseDown = function (event) {
        if (!this.isDescendant(this.el.nativeElement, event.target) &&
            event.target != this.directiveElementRef.nativeElement) {
            this.closeFontPicker();
        }
    };
    DialogComponent.prototype.onUploadFiles = function () {
    };
    DialogComponent.prototype.onCancelSelect = function () {
        this.selectedFont = false;
        this.font.family = this.initialFont.family;
        this.font.size = this.initialFont.size;
        this.font.files = this.initialFont.files;
        this.font.style = this.initialFont.style;
        this.font.styles = this.initialFont.styles;
        this.closeFontPicker();
    };
    DialogComponent.prototype.onSelectFont = function (font) {
        this.selectedFont = true;
        this.font.family = font.family;
        this.font.styles = font.styles;
        this.font.files = font.files;
        this.font.style = font.styles.indexOf("regular") > -1 ? "regular" : font.styles[0];
    };
    DialogComponent.prototype.onSearchReset = function (event) {
        this.searchTerm.setValue('');
        this.setCurentFonts(this.googleFonts);
    };
    DialogComponent.prototype.onFontStyleChange = function ($event, font) {
        var str = this.font.family + ":" + $event.srcElement.value;
        if (font.files) {
            WebFont.load({
                google: {
                    families: [str]
                }
            });
        }
    };
    DialogComponent.prototype.isDescendant = function (parent, child) {
        var node = child.parentNode;
        while (node !== null) {
            if (node === parent) {
                return true;
            }
            node = node.parentNode;
        }
        return false;
    };
    DialogComponent.prototype.createDialogBox = function (element, offset) {
        return {
            top: element.getBoundingClientRect().top + (offset ? window.pageYOffset : 0),
            left: element.getBoundingClientRect().left + (offset ? window.pageXOffset : 0),
            width: element.offsetWidth,
            height: element.offsetHeight
        };
    };
    DialogComponent.prototype.setDialogPosition = function () {
        var node = this.directiveElementRef.nativeElement, parentNode = null, position = 'static';
        while (node !== null && node.tagName !== 'HTML') {
            position = window.getComputedStyle(node).getPropertyValue("position");
            if (position !== 'static' && parentNode === null) {
                parentNode = node;
            }
            if (position === 'fixed') {
                break;
            }
            node = node.parentNode;
        }
        if (position !== 'fixed') {
            var boxDirective = this.createDialogBox(this.directiveElementRef.nativeElement, true);
            if (parentNode === null) {
                parentNode = node;
            }
            var boxParent = this.createDialogBox(parentNode, true);
            this.top = boxDirective.top - boxParent.top;
            this.left = boxDirective.left - boxParent.left;
        }
        else {
            var boxDirective = this.createDialogBox(this.directiveElementRef.nativeElement, false);
            this.top = boxDirective.top;
            this.left = boxDirective.left;
            this.position = 'fixed';
        }
        if (this.fpPosition === 'left') {
            this.top += boxDirective.height * this.fpPositionOffset / 100 - this.dialogArrowOffset;
            this.left -= this.fpWidth + this.dialogArrowSize;
        }
        else if (this.fpPosition === 'top') {
            this.top -= this.fpHeight + this.dialogArrowSize;
            this.left += this.fpPositionOffset / 100 * boxDirective.width - this.dialogArrowOffset;
            this.arrowTop = this.fpHeight - 1;
        }
        else if (this.fpPosition === 'bottom') {
            this.top += boxDirective.height + this.dialogArrowSize;
            this.left += this.fpPositionOffset / 100 * boxDirective.width - this.dialogArrowOffset;
        }
        else {
            this.top += boxDirective.height * this.fpPositionOffset / 100 - this.dialogArrowOffset;
            this.left += boxDirective.width + this.dialogArrowSize;
        }
    };
    __decorate([
        core_1.ViewChild(angular2_perfect_scrollbar_1.PerfectScrollbarComponent), 
        __metadata('design:type', angular2_perfect_scrollbar_1.PerfectScrollbarComponent)
    ], DialogComponent.prototype, "scrollbar", void 0);
    DialogComponent = __decorate([
        core_1.Component({
            selector: 'font-picker',
            template: require('dialog.component.html'),
            styles: [require('dialog.component.scss')]
        }), 
        __metadata('design:paramtypes', [core_1.ElementRef, font_picker_service_1.FontPickerService])
    ], DialogComponent);
    return DialogComponent;
}());
exports.DialogComponent = DialogComponent;
//# sourceMappingURL=dialog.component.js.map